@@ -90,7 +90,11 @@ pub fn delete_link<'a>(
                 Ok(header_hash)
             })
         }
-        _ => unreachable!(),
+        _ => Err(WasmError::Host(RibosomeError::HostFnPermissions(
+            call_context.zome.zome_name().clone(),
+            call_context.function_name().clone(),
+            "delete_link".into()
+        ).to_string()))
     }
 }
 
