@@ -36,7 +36,7 @@ const TIMEOUT_ERROR: &'static str = "inner function \'call_create_entry_remotely
 // test, especially when run in parallel with other tests.
 // This includes CI and many laptops.
 // @todo figure out why we can't have more than 4
-#[test_case(4)]
+// #[test_case(4)]
 // #[test_case(10)]
 fn conductors_call_remote(num_conductors: usize) {
     let f = async move {
